# Local long term should only be used for things that very rarely change
# and where it's okay for users to get old data.
# Invalidating this cache really doesn't do much since other
# instances of the server will have a different cache
LOCAL_LONG_TERM = "local_long_term"
